/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2002-2024, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Open Design Alliance software pursuant to a license 
//   agreement with Open Design Alliance.
//   Open Design Alliance Copyright (C) 2002-2024 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
///////////////////////////////////////////////////////////////////////////////
package com.opendesign.teigha.samples.servlet;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.opendesign.teigha.samples.java.BaseFileConverter;
import com.opendesign.teigha.samples.java.Converters;
import com.opendesign.teigha.samples.java.FileConverter;
import com.opendesign.teigha.samples.java.OdaSystem;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.MultipartConfig;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.servlet.http.Part;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.file.Files;
import java.util.Calendar;
import java.util.Formatter;
import java.util.Random;
import java.util.UUID;

@MultipartConfig(fileSizeThreshold = 10 * 1024 * 1024, // 10 MB
        maxFileSize = 10 * 1024 * 1024 * 5,   // 50 MB
        maxRequestSize = 10 * 1024 * 1024 * 10) // 100 MB
public class TeighaConverter extends HttpServlet {

    private String outputFilePath;
    private String inputFilePath;
    private boolean doKeepUploadedFiles = false;
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private final Gson gsonBuilder = new GsonBuilder().disableHtmlEscaping().create();

    public void init() {
        // Get the file location where it would be stored.
        outputFilePath = getInitParameter("output-file-storage");
        inputFilePath = getInitParameter("input-file-storage");
        doKeepUploadedFiles = "true".equalsIgnoreCase(getInitParameter("do-keep-uploaded-files"));
        BaseFileConverter.init();
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {

        try {
            Part thePart = request.getPart("filename");

            File localInputFile = getFilePath(inputFilePath);
            localInputFile.getParentFile().mkdirs();
            logger.info("Uploading file to {}", localInputFile);

            Files.copy(thePart.getInputStream(), localInputFile.toPath());

            File outFile = getFilePath(outputFilePath);
            outFile.getParentFile().mkdirs();

            File outFilePdf = new File(outFile + ".pdf");
            File outFilePng = new File(outFile + ".png");

            synchronized (this) {
                FileConverter pdfConverter = getFileConverter("pdf");
                logger.info("Converting file {} to {}", localInputFile, outFilePdf);
                pdfConverter.convertFile(localInputFile, outFilePdf);

                FileConverter pngConverter = getFileConverter("png");
                logger.info("Converting file {} to {}", localInputFile, outFilePng);
                pngConverter.convertFile(localInputFile, outFilePng);
            }

            String uuid = UUID.randomUUID().toString();

            request.getSession(false).setAttribute(uuid + ".pdf", outFilePdf.getAbsolutePath());
            request.getSession(false).setAttribute(uuid + ".png", outFilePng.getAbsolutePath());

            // return result
            String uri = request.getRequestURI();
            String ctxPath = request.getContextPath();
            String pngUrl = String.format("%s?to=png&result=%s.png", uri, uuid);
            String pdfUrl = String.format("%s?to=pdf&result=%s.pdf", uri, uuid);
            String resUrl = String.format("%s?result=%s", ctxPath, uuid);
            String version = OdaSystem.hostApp.product() + " " + OdaSystem.hostApp.versionString();
            ProcessingResult pr = new ProcessingResult(pngUrl, pdfUrl, resUrl, version); // ###
            String json = gsonBuilder.toJson(pr);
            PrintWriter out = response.getWriter();
            response.setContentType("application/json");
            response.setCharacterEncoding("UTF-8");
            out.print(json);
            out.flush();

            if (!doKeepUploadedFiles) localInputFile.delete();

        } catch (Exception ex) {
            ex.printStackTrace();
            throw new ServletException("Can't process input file", ex);
        }
    }

    private File getFilePath(String parentFolder) throws IOException {
        Formatter f = new Formatter();
        Calendar c = Calendar.getInstance();
        f.format("%s/%d/%02d/%02d/%d.dwg", parentFolder, c.get(Calendar.YEAR), c.get(Calendar.MONTH) + 1,
                c.get(Calendar.DAY_OF_MONTH), Math.abs(new Random().nextLong()));
        return new File(f.toString());
    }

    private FileConverter getFileConverter(String outFormat) throws ServletException {
        FileConverter converter = Converters.getInstance().get(outFormat);
        if (null == converter)
            throw new ServletException("Incorrect output file format or required converter is not found: " + outFormat);
        return converter;
    }

    /**
     * @param outFormat
     * @return
     */
    private String outFormat(String outFormat) {
        if (outFormat == null || "null".equals(outFormat)) outFormat = "pdf";
        return outFormat;
    }

    /**
     * @param request
     * @param response
     * @throws ServletException
     * @throws java.io.IOException
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {

        String uuid = request.getParameter("result");
        String result = (String) request.getSession(false).getAttribute(uuid);

        if (result == null) {

            throw new ServletException("GET parameter 'result' is not set");

        } else {

            File file = new File(result);

            String outFormat = outFormat(request.getParameter("to"));
            FileConverter converter = getFileConverter(outFormat);

            String disposition = "inline; filename=\"" + uuid + "." + converter.getFileExtension() + "\"";
            response.setHeader("Content-disposition", disposition);

            response.setContentType(converter.getContentType());
            response.setContentLength((int) file.length());

            OutputStream outStream = response.getOutputStream();
            FileInputStream inStream = new FileInputStream(file);
            IOUtils.copy(inStream, outStream);

            inStream.close();
        }
    }

    private static class ProcessingResult {
        private String pngUrl;
        private String pdfUrl;
        private String resultPageUrl;
        private String version;

        public String getPngUrl() {
            return pngUrl;
        }

        public String getPdfUrl() {
            return pdfUrl;
        }

        public String getResultPageUrl() {
            return resultPageUrl;
        }

        public String getVersion() { return version; }

        public ProcessingResult(String pngUrl, String pdfUrl, String resultPageUrl, String version) {
            this.pngUrl = pngUrl;
            this.pdfUrl = pdfUrl;
            this.resultPageUrl = resultPageUrl;
            this.version = version;
        }
    }
}
